#ifndef _CONTACT_H_
#define _CONTACT_H_

#include <wtypes.h>
#include <malloc.h>
#include <crtdbg.h>
#include <conio.h>
#include <stdio.h>

#define ItemSize 256 


/////////////////////////////////////////////////////////////////////////
//Class: Allocator
//Descr: allocator pameti, v contructoru si naalokuje pamet, pokud skonci
//       tak alokuje dalsi za behu programu

template<WORD BlockSize = 25600>
class Allocator 
{
	
public:
		
	Allocator():m_block(0)
	{
		_ASSERT( BlockSize < WORD(-1) );	
		CreateBlock();
	}
	
	~Allocator()
	{
		FreeAll();
	}
  
  //return avaible memory or alocate memory block
	void * Alloc( WORD wSize )
	{
		
		if(! m_block || wSize > m_block->m_free )
		{
			if( !CreateBlock() )
				return false;
		}
		
		void * data			=	m_block->m_data;
		m_block->m_data		+=	wSize;
		m_block->m_free		-=	wSize;
		
		return data;
	}

  //not used 
  void Dealoc()
  {}
	
  //free all memory
	void FreeAll()
	{
		while ( m_block )
		{
			Block * tmp = m_block->prev;
			free ( m_block );
			m_block = tmp;
		}
	}
	
protected:
	//////////////////////////////////////////////////////////////////////////
	//Function:CreateBlock()
	//Purpose	:Create new block or return availble
	//return  :bool - true if success, false if failed
	bool CreateBlock()
	{
		Block * tmp =  (Block *) malloc( BlockSize + sizeof(Block) );
		if( !tmp )
			return false;
		
		tmp->m_free		= BlockSize;
		tmp->m_data		= (BYTE*)tmp + sizeof( Block );
		ZeroMemory( tmp->m_data, BlockSize );
		tmp->next		= 0;
		tmp->prev		= m_block;
		if( m_block )
			m_block->next	= tmp;
			m_block			= tmp;
				
		return true;
	}
	
	
private:	
	struct Block
	{
		Block * prev;	//previous block
		Block * next;	//next block - not used yet
		WORD	m_free; //number of free bytes
		PBYTE	m_data; //useful data
	};
		
	Block *		m_block;//pointer to the head of Blocks
};



/////////////////////////////////////////////////////////////////////////
//Class: Contact
//Descr: class for easier handling with data
class Contact
{
public:
	
	struct Data
	{
		PCSTR	pcsName;	//name
		BYTE	bSize;		//velikost
		BYTE	bIdx;		//pozice v poli
	};

	static Data s_data[];
	
	enum // enum of data to be stored
	{
		Begin	= 0,
		Name	= Begin,
  		Surname	,
		Icq	,
		Email ,
		Total	= Email,
	};


	Contact(){}

	void * operator new( unsigned int uiSize )
	{
		return mem.Alloc( ItemSize );
	}
		
	//leave it free, every thing will be destroy in the descrutor
	void   operator delete(void * data){}
	
	
	//return parameter according bName(enum)
	PCSTR GetParam( BYTE bName ) const
	{		
		_ASSERT( bName <=Total );
		return (const char *) (m_data+ s_data[bName].bIdx) ;
	}
	
	
	//set data into Contact 
	void SetParam( BYTE bName, PCSTR str )
	{
		_ASSERT( bName <= Total );
		if( strlen(str) < s_data[bName].bSize )
			memcpy(&m_data[s_data[bName].bIdx],str, strlen(str) );
	}

  //jednoduche menu
  static BYTE Menu()
  {
    for( int i = 0 ;  i <= Total; i++ )
    {
      printf("\n %d - %s", i, s_data[i].pcsName);
    }
    return getch();
  }
  


protected:
	
/*	//helper function
	void * End( void* pData, int & len )
	{
		BYTE * data = (BYTE*)pData;

		for( len = 0; len < ItemSize; len++ )
		{
			if( *( data + len ) == ';' )
			{
				return (void *)( data+len+1 );
			}	
		}
		
		return 0;
	}
*/
private:
	static  Allocator<> mem;		//alocator use default value
	BYTE				m_data[1];	//useful data
};

#endif
